#include "lcd.h"

#include <avr/io.h>
#include <stdio.h>
#include <stdlib.h>
#include <util/delay.h>

/*
 * wait_ms(..) - warte fuer x ms
 */
void wait_ms(int ms) 
{
	_delay_loop_2( 1*(F_CPU/(1000/4)) * ms);	// 4 Zyklen Warteschleife
}

/* 
 * lcd_send(..) - sendet ein Byte an LCD im 4-Bit-Modus
 * nicht direkt aufrufen sondern lcd_cmd oder lcd_write nutzen
 */
void lcd_send(char data)
{
	// aktuelles RS ermitteln
	char rs=PORTD;
	rs&=4;
	// High-Teil senden
	char tmp=data;
	tmp&=0xf0;
	tmp|=rs;
	PORTD=tmp;
	// Schreibsignal
	PORTD |= (1<<3);	//sbi(PORTD,3);
	PORTD &= ~(1<<3);	//cbi(PORTD,3);
	// Low-Teil senden
	tmp=data;
	tmp&=0x0f;
	tmp*=16;
	tmp|=rs;
	PORTD=tmp;
	// Schreibsignal
	PORTD |= (1<<3);	//sbi(PORTD,3);
	PORTD &= ~(1<<3);	//cbi(PORTD,3);
	// verarbeiten lassen
	wait_ms(1);
}

/*
 * lcd_cmd(..) - sendet ein Kommando an LCD
 */
void lcd_cmd(char cmd)
{
	PORTD &= ~(1<<2);	//cbi(PORTD,2); RS loeschen = Kommando
	lcd_send(cmd);		// senden
}

/*
 * lcd_write(..) - sendet ein Zeichen an LCD
 */
void lcd_write(char c)
{
	PORTD |= (1<<2);	//sbi(PORTD,2);	// RS setzen = Daten
	lcd_send(c);		// senden
}

/*
 * lcd_write(..) - sendet eine Zeichenkette an LCD
 */
void lcd_write_s(char *buf)
{
	while(buf[0]!='\0') {
		lcd_write(buf[0]);
		buf++;
	}
}

/*
 * lcd_home(..) - Cursor auf Position 1,1
 */
void lcd_home(void)
{
	lcd_cmd(LCD_MOVE_CURSOR_HOME);
	wait_ms(2);
}
/*
 * lcd_clear(..) - loescht die Anzeige im LCD
 */
void lcd_clear(void)
{
	lcd_cmd(LCD_DISP_CLEAR);
	wait_ms(2);
}

/*
 * lcd_on(..) - schaltet das LCD und den Cursor an
 */
void lcd_on(void)
{
	lcd_cmd(LCD_DISP_ON);
}

/*
 * lcd_off(..) - schaltet das LCD aus
 */
void lcd_off(void)
{
	lcd_cmd(LCD_DISP_OFF);
}

/*
 * lcd_goto(..) - setzt die Cursorposition
 */
void lcd_goto(int row, int col)
{
	row--;			// Null-basierend
	row&=0x01;		// sicherheitshalber
	row*=0x40;		// Zeile nach Bit 6 bringen
	col--;			// Null-basierend
	col&=0x0f;		// sicherheitshalber
	char tmp=row|col;
	tmp|=0x80;		// Cursor setzen
	lcd_cmd(tmp);		// senden
}

/*
 * lcd_init(..) - Schaltet die Ports und Initialisiert das LCD
 */
void lcd_init(void)
{
	// Port D = Ausgang
	DDRD=0xff;
	PORTD=0;
	// warten bist LCD-Controller gebootet
	wait_ms(200);		
	// 4-BitModus einschalten
	PORTD=0x20;			
	// Schreibsignal
	PORTD |= (1<<3);						//sbi(PORTD,3);
	PORTD &= ~(1<<3);						//cbi(PORTD,3);
	wait_ms(5);			// Zeit zum Umschalten lassen
	// ab hier im 4-Bit-Modus
	lcd_cmd(0x28);		// Funktions-Set: 2 Zeilen, 5x7 Matrix, 4 Bit
	//lcd_off();
	lcd_cmd(0x06);		// Entry Mode
	lcd_on();
	lcd_clear();
}

/*
 * lcd_charDef(..) - initialisiert benutzerdefinierte Zeichen fr das LCD
 */
void lcd_charDef(void)
{
	// 1. Kommando CGRAM + Adresse setzen
	// 2. Punktmatrix-Daten laden
	unsigned char charSet[64]={	
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,
		0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,
		0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,
		0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,
		0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,
		0x00,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
		0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
		0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
	};
	int i=0;
	for (i=0;i<64;i++) {
		lcd_cmd(0x40+i);	// cmd 0x40 + Adr
		lcd_write(charSet[i]);	// Pixel
	}
}

// Nummer an den LCD schreiben, immer 2 Stellen, wenn kleiner als 10 dann 0 vorranstellen
void lcd_write_num_l(int numb) {
		//char *buffer[2];
		char buffer[2];	
		if(numb < 10) {
			lcd_write_s("0");
		} 
		
		itoa(numb, buffer, 10);
		lcd_write_s(buffer);
}

